/*
 * cmms - Car MultiMedia System
 * by Tobias Weihmann <tobias@fomalhaut.de>
 * and Janosch Rolles <janosch@base2.de>
 *
 * For detailed information view cmms.c and
 * project documentation in usr/doc of the
 * distribution or visit http://www.base2.de
 * or send email to cmms@base2.de
 *
 * 17.3.02 ftw: wenn nach einem eintrag gefragt wurde, der nicht in der
 *         opl ist, und die parameter device, liste, song waren, dann 
 *         wird nicht mehr auf den ersten song der opl gesetzt, sondern
 *         auf die eingabeparameter. die songid wird dann nicht geschrieben
 *         (sonst springt er bei einem ungueltigen gotosong immer auf die
 *         erste liste.)
 */

/* askopl searches a song id in the OrderPlayList and writes DEVICE, LIST and SONG */

#define OPL_FILE "/tmp/OPL"

#define DEVICE_FILE "/tmp/DEVICE"
#define LIST_FILE "/tmp/LIST"
#define SONG_FILE "/tmp/SONG"

#include <stdio.h>
#include <dirent.h>

int main(int argc, char **argv) {
    FILE *opl;
    FILE *devicefile;
    FILE *listfile;
    FILE *songfile;

    char waste[NAME_MAX];
    int songid,dev,list,song,success=0;
    int sdev, slist, ssong;

    if (argc==2 || argc==4) {
        opl = fopen(OPL_FILE, "r");
        if (opl == NULL) {
            fprintf(stderr, "Could not open OPL file <%s>.\n", OPL_FILE);
            exit(2);
        }

        if (argc==2) {
            int ssongid=atoi(argv[1]);
            while (!feof(opl) && success==0) {
                fscanf(opl, "%d %d %d %d\n", &songid, &dev, &list, &song);
                if (songid==ssongid) {
                    success=1;
                }
            }
        }

        if (argc==4) {
            sdev=atoi(argv[1]);
            slist=atoi(argv[2]);
            ssong=atoi(argv[3]);
            while (!feof(opl) && success==0) {
                fscanf(opl, "%d %d %d %d\n", &songid, &dev, &list, &song);
                if (dev==sdev && list==slist && song==ssong) {
                    success=1;
                }
            }
        }

        fclose(opl);

        if (success != 1) {
            if (argc==4) { 
               // Wenn nicht erfolgreich, moeglichst alte Werte behalten.
               song = ssong;
               dev = sdev;
               list = slist;
            } else {
               // haben nach Song-ID gesucht, also erster Eintrag der OPL
               opl = fopen(OPL_FILE, "r");
               if (opl == NULL) {
                   fprintf(stderr, "Could not open OPL file <%s>.\n", OPL_FILE);
                   exit(2);
               }
               fscanf(opl, "%d %d %d %d\n", &songid, &dev, &list, &song);
            }
        }

        /* print out songid */
        if (success == 1 || argc == 2) printf("%i\n",songid);
        /* save results */
        devicefile = fopen(DEVICE_FILE,"w");
        listfile = fopen(LIST_FILE,"w");
        songfile = fopen(SONG_FILE,"w");
        if (devicefile == NULL || listfile == NULL || songfile == NULL) {
            fprintf(stderr, "Could not open output files: <%s>, <%s>, <%s>.\n", DEVICE_FILE, LIST_FILE, SONG_FILE);
            exit(2);
        }
        fprintf(devicefile,"%i\n",dev);
        fprintf(listfile,"%i\n",list);
        fprintf(songfile,"%i\n",song);
        fflush(devicefile);
        fflush(listfile);
        fflush(songfile);
        fclose(devicefile);
        fclose(listfile);
        fclose(songfile);
    } else {
        fprintf(stderr,"Usage: askopl <opl-song-id>\n");
        fprintf(stderr,"   or: askopl <device-nr> <list-nr> <song-nr>\n");
        exit(2);
    }

    return 0;
}
