/*
 * cmms - Car MultiMedia System
 * by Tobias Weihmann <tobias@fomalhaut.de>
 * and Janosch Rolles <janosch@base2.de>
 *
 * ID3-tag code taken from Christian Trster <c.troester@gmx.de>
 *
 * For detailed information view cmms.c and
 * project documentation in usr/doc of the
 * distribution or visit http://www.base2.de
 * or send email to cmms@base2.de
 *
 */

#define MAX_BUFFER 655
#define DEBUG 0

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void catchtime();
void catchtag();
void write_time();
void write_title();
void save_framepos();
void player_restart();
void remove_spaces();


FILE *displayfifo = NULL;
char input[MAX_BUFFER];
int lastsecgone=0, seccounter=0;

int main(int argc, char **argv) {
    displayfifo = fopen("/tmp/fifo.display","w");
    if (displayfifo == NULL) {
        fprintf(stderr,"[catchmsg]error opening displayfifo\n");
        return 1;
    }
    while (!feof(stdin)) {
        fscanf(stdin, "\n%[^\n]",input);
        if (strncmp(input,"@I",2)==0) {
            catchtag();
        }
        if (strcmp("@P 0",input)==0) {
            save_framepos(0);
            system("nextsong.sh STOPCAUGHT>/tmp/fifo.player");
            //why??//system("/bin/sleep 2");
        } else {
            catchtime();
        }
        printf("%s\n",input);
        strcpy(input,"\0");
    }
    fclose(displayfifo);
    // make the player to be restarted if it crashed
    player_restart();
    return 0;
}

void catchtime() {
    char wastestr[1], framegonestr[17]="", frameleftstr[17]="", secgonestr[9]="", secleftstr[9]="";
    if (sscanf(input, "@F %16[1234567890] %16[1234567890] %8[1234567890].%0s %8[1234567890].%0s", framegonestr, frameleftstr, secgonestr, wastestr, secleftstr, wastestr) == 6) {
        int secgone, secleft, framegone;
        secgone = atoi(secgonestr);
        secleft = atoi(secleftstr);
        framegone = atoi(framegonestr);
        if (secgone != lastsecgone) {
            int mingone, minleft;
            char writestr[255];
            lastsecgone = secgone;
            /* save framepos every DELAY secs */
            seccounter++;
            if (seccounter >= 10) {
                seccounter = 0;
                save_framepos(framegone);
            }
            /* send time to cmms_display process */
            mingone = secgone / 60; minleft = secleft / 60;
            secgone = secgone % 60; secleft = secleft % 60;
            while (mingone > 99) mingone -= 100;
            while (minleft > 99) minleft -= 100;
            sprintf(writestr,"+%02i:%02i -%02i:%02i",mingone, secgone, minleft, secleft);
            write_time(writestr);
        }
    }
}

void catchtag() {
    char id3tagtitlestr[128]="\0";
    char title[31]="", artist[31]="", album[31]="", year[5]="", comment[31]="", genre[11]="";
    // for now only the title and the artist are being caught
    if(sscanf(input, "@I ID3:%30c%30c%30c%4c%30c%s", title, artist, album, year, comment, genre) == 6) {
        title[31]   = '\0';
        artist[31]  = '\0';
        album[31]   = '\0';
        year[4]     = '\0';
        comment[31] = '\0';
        genre[11]   = '\0';

        remove_spaces(title);
        remove_spaces(artist);

        sprintf(id3tagtitlestr,"%s - %s\0",artist,title);
#ifdef DEBUG
        fprintf(stdout,"[catchmsg]id3tagtitle: >%s<\n",id3tagtitlestr);
        fprintf(stdout,"title:>%s<\nauthor>%s<\nalbum>%s<\nyear>%s<\ncomment>%s<\ngenre>%s<\n\n", title, artist, album, year, comment, genre);
#endif
    } else {
        strcpy(id3tagtitlestr,"(no-id3-tag)\0");
    }
    write_title(id3tagtitlestr);
}

void write_time(char *writestr) {
    fprintf(displayfifo,"display_settime %13s\n",writestr);
    fflush(displayfifo);
}

void write_title(char *writestr) {
    fprintf(displayfifo,"display_setid3tagtitle %s\n",writestr);
    fflush(displayfifo);
}

void save_framepos(int pos) {
    FILE *frameposfile = NULL;
    frameposfile = fopen("/tmp/FRAMEPOS","w");
    if (frameposfile == NULL) {
        fprintf(stderr,"[catchmsg]error opening frameposfile\n");
    }
    fprintf(frameposfile,"%16d\n",pos);
    fflush(frameposfile);
    fclose(frameposfile);
    system("sync");
}

void player_restart() {
    system("killall -USR2 cmms");
}

void remove_spaces(char *string) {
    int i;
    for (i=strlen(string)-1;i>=0;i--) {
        if ((string[i] != ' ') || (i==0 && (string[0] == ' '))) {
            string[i+1]= '\0';
            i = 0;
        }
    }
}

