/*
 * cmms - Car MultiMedia System
 * by Tobias Weihmann <tobias@fomalhaut.de>
 * and Janosch Rolles <janosch@base2.de>
 *
 * For detailed information view cmms.c and
 * project documentation in usr/doc of the
 * distribution or visit http://www.base2.de
 * or send email to cmms@base2.de
 *
 */

/* _Includes_ */
#include <stdio.h>
#include <getopt.h>
#include <unctrl.h>
#include <vga.h>
#include <vgakeyboard.h>
#include <signal.h>
#include <sys/time.h>
#include <stdlib.h>

/* _Definitions_ */
/* command line options */
#define OPF_WHEEL   'w'
#define OPF_DEBUG   'd'
#define OPF_HELP    'h'
/* command line options for partner processes */
#define OPF_LCD     'l'
#define OPF_ID3TAG  't'
/* keyboard modi */
#define KEYBOARD_MODECONTROL    0
#define KEYBOARD_MODENUMINPUT   1
#define KEYBOARD_MODEMIXER      2

/* _Variables_ */
/* command line options */
bool wheel;
bool debug;
/* command line options for partner processes */
bool lcd;
bool id3tag;
/* Inter Process Communication */
FILE *controlfifo = NULL;
FILE *displayfifo = NULL;
/* execution */
bool running;
int key_repeated;
/* input */
int keyboard_timeoutcounter;
int keyboard_mode;
int keyboard_mode_max;
int wheel_direction;
bool wheel_executed;
/* Numinput */
int numinput_digits;
int numinput_value;

/* _Functions_ */
void usage();
void debugmsg();
void ipc_startprograms();
void ipc_startplayer();
void ipc_restartplayer();
void ipc_init();
void ipc_close();
void ipc_displaycommand();
void ipc_controlcommand();
void keyboard_starttimer();
void keyboard_signals();
void keyboard_getinput();
void keyboard_key();
void keyboard_actionboth();
void keyboard_actiononce();
void keyboard_actionrepeated();
void keyboard_allmodesboth();
void keyboard_allmodesonce();
void keyboard_allmodesrepeated();
void keyboard_controlboth();
void keyboard_controlonce();
void keyboard_controlrepeated();
void keyboard_numinputboth();
void keyboard_numinputonce();
void keyboard_numinputrepeated();
void keyboard_mixerboth();
void keyboard_mixeronce();
void keyboard_mixerrepeated();
void keyboard_setmode();
void collect_digit();
void wheel_getinput();
void wheel_action();
