/*
 * cmms - Car MultiMedia System
 * by Tobias Weihmann <tobias@fomalhaut.de>
 * and Janosch Rolles <janosch@base2.de>
 *
 * For detailed information view cmms.c and
 * project documentation in usr/doc of the
 * distribution or visit http://www.base2.de
 * or send email to cmms@base2.de
 *
 */

#define MPL_FILE "/tmp/MPL"
#define OPL_FILE "/tmp/OPL"
#define MAXLENGTH 99999

/* command line options */
#define OPF_RANDOM_ALL  'a'
#define OPF_RANDOM_DEV  'd'
#define OPF_RANDOM_LIST 'l'
#define OPF_HELP        'h'

#define RANDOM_OFF      0
#define RANDOM_ALL      1
#define RANDOM_DEV      2
#define RANDOM_LIST     3

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/timeb.h>

void usage();

int main(int argc, char *argv[]) {
    /* common */
    FILE *mpl;
    FILE *opl;
    int src_dev[MAXLENGTH],src_list[MAXLENGTH],src_song[MAXLENGTH];
    int trgt_dev[MAXLENGTH],trgt_list[MAXLENGTH],trgt_song[MAXLENGTH];
    int current_buffer;
    int i, entries, current_entries_begin, current_entries_end;
    /* for randomic number generator */
    int target;
    short seed;
    struct timeb tp;
    /* Handle command line args */
    int optch;
    char cmdopts[] = { OPF_RANDOM_ALL, OPF_RANDOM_DEV, OPF_RANDOM_LIST, OPF_HELP };
    int random_mode = RANDOM_OFF;
    while( (optch = getopt(argc,argv,cmdopts)) != -1 ) {
        switch (optch) {
            case OPF_RANDOM_ALL:
                random_mode = RANDOM_ALL;
                break;
            case OPF_RANDOM_DEV:
                random_mode = RANDOM_DEV;
                break;
            case OPF_RANDOM_LIST:
                random_mode = RANDOM_LIST;
                break;
            case OPF_HELP:
                usage();
                return 1;
                break;
            default:
                usage();
                return 1;
                break;
        }
    }
    /* read MasterPlayList */
    mpl = fopen(MPL_FILE, "r");
    if (mpl == NULL) {
        fprintf(stderr, "[createopl]Could not open MasterPlayList file <%s>.\n", MPL_FILE);
        exit(2);
    }
    i = 0;
    while (!feof(mpl) && i<=MAXLENGTH) {
        int tmp_dev,tmp_list,tmp_song;
        char tmp_filename[NAME_MAX];
        fscanf(mpl,"%d%d%d %[^\n]", &tmp_dev, &tmp_list, &tmp_song, tmp_filename);
        if (tmp_dev != 0 && tmp_list != 0 && tmp_song != 0) {
            src_dev[i] = tmp_dev;
            src_list[i] = tmp_list;
            src_song[i] = tmp_song;
            i++;
        }
    }
    entries = i;
    fprintf(stderr, "[createopl]found %i entries in MasterPlayList <%s>.\n", entries, MPL_FILE);
    fclose(mpl);
    /* generate OrderPlayList */
    opl = fopen(OPL_FILE, "w");
    if (opl == NULL) {
        fprintf(stderr, "[createopl]Could not open OrderPlayList file <%s> for writing.\n", OPL_FILE);
        exit(2);
    }
    switch(random_mode) {
        case RANDOM_OFF:
            for (i=0;i<entries;i++) {
                trgt_dev[i] = src_dev[i];
                trgt_list[i] = src_list[i];
                trgt_song[i] = src_song[i];
                //fprintf(opl,"%i %i %i %i\n",i+1,odev[i],olist[i],osong[i]);
            }
            entries = i;
            fprintf(stderr, "[createopl]buffered %i entries in SEQUENTIAL order.\n", entries);
            break;
        case RANDOM_ALL:
            /* mark all entries as empty */
            for (i=0;i<entries;i++) {
                trgt_dev[i] = trgt_list[i] = trgt_song[i] = -1;
            }
            /* fill them up in randomized order */
            fprintf(stderr,"[createopl]filling up playlist positions in randomized order... please wait...\n",i);
            i = 0;
            while (i<entries) {
                /* get end of entries for current list
                 * (current_entries_end is the first entry of the next device) */
                current_entries_begin = i;
                current_entries_end = entries;
                //DEBUG//fprintf(stderr,"[createopl]current list has entries from %i to %i...\n",i,current_entries_end);
                /* fill them up in randomized order */
                while (i<current_entries_end) {
                    int z;
                    /* generate randomic target position
                     * (0 <= target < free positions of current entries) */
                    ftime(&tp);
                    /* we multiply the seed (on fast machines we would get the same seed if
                     * it can manage several cycles in 1ms */
                    seed = tp.millitm * (short)(i+1);
                    srand(seed);
                    target = (int) (((float)(current_entries_end-i))*rand()/(RAND_MAX+1.0));
                    //DEBUG//fprintf(stderr,"[createopl]free positions: %i target: %i\n",current_entries_end-i,target);
                    /* the target just refers to the empty entries - get the real target */
                    target = target + current_entries_begin;
                    for (z=current_entries_begin;z<=target;z++) {
                        if (trgt_dev[z] != -1) {
                            target++;
                        }
                    }
                    //DEBUG//fprintf(stderr,"[createopl]final target pos: %i\n",target);
                    /* write to target */
                    trgt_dev[target] = src_dev[i];
                    trgt_list[target] = src_list[i];
                    trgt_song[target] = src_song[i];
                    i++;
                    /*
                    if (i%25==0 || i>=entries) {
                        fprintf(stdout,"display_settitle Randomize: (%i of %i)\n",i,entries);
                    }
                    */
                }
            }
            entries = i;
            fprintf(stderr, "[createopl]buffered %i entries in RANDOMIZED order (all).\n", entries);
            break;
        case RANDOM_DEV:
            /* mark all entries as empty */
            for (i=0;i<entries;i++) {
                trgt_dev[i] = trgt_list[i] = trgt_song[i] = -1;
            }
            /* fill them up in randomized order */
            fprintf(stderr,"[createopl]filling up playlist positions in randomized order... please wait...\n",i);
            i = 0;
            while (i<entries) {
                /* get end of entries for current list
                 * (current_entries_end is the first entry of the next device) */
                current_entries_end = current_entries_begin = i;
                while (current_entries_end<entries && src_dev[current_entries_end]==src_dev[current_entries_begin]) {
                    current_entries_end++;
                }
                //DEBUG//fprintf(stderr,"[createopl]current list has entries from %i to %i...\n",i,current_entries_end);
                /* fill them up in randomized order */
                while (i<current_entries_end) {
                    int z;
                    /* generate randomic target position
                     * (0 <= target < free positions of current entries) */
                    ftime(&tp);
                    /* we multiply the seed (on fast machines we would get the same seed if
                     * it can manage several cycles in 1ms */
                    seed = tp.millitm * (short)(i+1);
                    srand(seed);
                    target = (int) (((float)(current_entries_end-i))*rand()/(RAND_MAX+1.0));
                    //DEBUG//fprintf(stderr,"[createopl]free positions: %i target: %i\n",current_entries_end-i,target);
                    /* the target just refers to the empty entries - get the real target */
                    target = target + current_entries_begin;
                    for (z=current_entries_begin;z<=target;z++) {
                        if (trgt_dev[z] != -1) {
                            target++;
                        }
                    }
                    //DEBUG//fprintf(stderr,"[createopl]final target pos: %i\n",target);
                    /* write to target */
                    trgt_dev[target] = src_dev[i];
                    trgt_list[target] = src_list[i];
                    trgt_song[target] = src_song[i];
                    i++;
                    /*
                    if (i%25==0 || i>=entries) {
                        fprintf(stdout,"display_settitle Randomize: (%i of %i)\n",i,entries);
                    }
                    */
                }
            }
            entries = i;
            fprintf(stderr, "[createopl]buffered %i entries in RANDOMIZED order (device).\n", entries);
            break;
        case RANDOM_LIST:
            /* mark all entries as empty */
            for (i=0;i<entries;i++) {
                trgt_dev[i] = trgt_list[i] = trgt_song[i] = -1;
            }
            /* fill them up in randomized order */
            fprintf(stderr,"[createopl]filling up playlist positions in randomized order... please wait...\n",i);
            i = 0;
            while (i<entries) {
                /* get end of entries for current list
                 * (current_entries_end is the first entry of the next device) */
                current_entries_end = current_entries_begin = i;
                while (current_entries_end<entries && src_list[current_entries_end]==src_list[current_entries_begin]) {
                    current_entries_end++;
                }
                //DEBUG//fprintf(stderr,"[createopl]current list has entries from %i to %i...\n",i,current_entries_end);
                /* fill them up in randomized order */
                while (i<current_entries_end) {
                    int z;
                    /* generate randomic target position
                     * (0 <= target < free positions of current entries) */
                    ftime(&tp);
                    /* we multiply the seed (on fast machines we would get the same seed if
                     * it can manage several cycles in 1ms */
                    seed = tp.millitm * (short)(i+1);
                    srand(seed);
                    target = (int) (((float)(current_entries_end-i))*rand()/(RAND_MAX+1.0));
                    //DEBUG//fprintf(stderr,"[createopl]free positions: %i target: %i\n",current_entries_end-i,target);
                    /* the target just refers to the empty entries - get the real target */
                    target = target + current_entries_begin;
                    for (z=current_entries_begin;z<=target;z++) {
                        if (trgt_dev[z] != -1) {
                            target++;
                        }
                    }
                    //DEBUG//fprintf(stderr,"[createopl]final target pos: %i\n",target);
                    /* write to target */
                    trgt_dev[target] = src_dev[i];
                    trgt_list[target] = src_list[i];
                    trgt_song[target] = src_song[i];
                    i++;
                    /*
                    if (i%25==0 || i>=entries) {
                        fprintf(stdout,"display_settitle Randomize: (%i of %i)\n",i,entries);
                    }
                    */
                }
            }
            entries = i;
            fprintf(stderr, "[createopl]buffered %i entries in RANDOMIZED order (list).\n", entries);
            break;
        default:
            fprintf(stderr, "[createopl]Ooups! You just hit a bug in the code (it shouldn't be possible to reach this part of code).\n");
            return 1;
            break;
    }
    /* write OrderPlayList file */
    for (i=0;i<entries;i++) {
        fprintf(opl,"%i %i %i %i\n",i+1,trgt_dev[i],trgt_list[i],trgt_song[i]);
    }
    entries = i;
    fprintf(stderr, "[createopl]wrote %i entries to OrderPlayList <%s>.\n", entries, OPL_FILE);
    fflush(opl);
    fclose(opl);
    /* exit */
    return 0;
}

void usage() {
    /* print help */
    fprintf(stderr, "[createopl]usage: 'createopl [-a|-d|-l]'.\n");
    fprintf(stderr, "[createopl]with random mode: -a (all), -d (device) or -l (list).\n");
}
